/* Xlrem Login Reminder program for Motif
 * Copyright (C) 1997 George Lebl.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "xlrem.h"

extern XtAppContext	app;

extern char *reminders[];
extern int remcount;

/*
 * Message Box Callbacks
 */
void
mbox_callback(Widget dialog, XtPointer client_data, XtPointer call_data)
{
	*((int *)(client_data))=1;
}
void
mboxdes_callback(Widget dialog, XtPointer client_data, XtPointer call_data)
{
	*((int *)(client_data))=2;
}

/*
 * Message Box, simple dialog only with ok and a message
 * only created once
 * APLICATION MODAL
 */
void
mbox(Widget parent, char *msg, char *tit)
{
	static Widget dialog=NULL;
	XmString text;
	XmString title;
	static int done=0;
	Arg args[10];
	int n;

	if(!dialog) {
		n=0;
		XtSetArg(args[n],XmNdeleteResponse,XmDESTROY);n++;
		XtSetArg(args[n],XmNautoUnmanage,False);n++;
		XtSetArg(args[n],XmNdialogStyle,
			XmDIALOG_FULL_APPLICATION_MODAL);n++;
		XtSetArg(args[n],XmNdialogType,XmDIALOG_INFORMATION);n++;
		dialog = XmCreateMessageDialog(parent, "dialog", args, n);
		XtUnmanageChild(
			XmMessageBoxGetChild(dialog,XmDIALOG_CANCEL_BUTTON));
		XtUnmanageChild(
			XmMessageBoxGetChild(dialog,XmDIALOG_HELP_BUTTON));
		XtAddCallback(dialog,XmNokCallback,mbox_callback,&done);
		XtAddCallback(dialog,XmNdestroyCallback,mboxdes_callback,&done);
	}
	text=XmStringCreateLocalized(msg);
	title=XmStringCreateLocalized(tit);
	XtVaSetValues(dialog,
		XmNmessageString,text,
		XmNdialogTitle,title,
		NULL);
	XmStringFree(text);
	XmStringFree(title);

	done=0;
	XtManageChild(dialog);
	XtPopup(XtParent(dialog),XtGrabNone);
	while(!done)
		XtAppProcessEvent(app, XtIMAll);
	/*if it wasn't destroyed close it*/
	if(done!=2) {
		XtPopdown(XtParent(dialog));
	/*if it was destroyed set dialog to NULL so*/
	/*that we can use this dialog again*/
	} else {
		dialog=NULL;
	}
}

/*
 * Read the reminders file
 */
int
read_rem_file(char filename[])
{
	FILE *fp;
	char tmp[MAXREMLEN];
	char *sc;

	if(!(fp=fopen(filename,"rt")))
		return 0;

	while(fgets(tmp,MAXREMLEN,fp)) {
		if((sc=strchr(tmp,'\n'))) *sc='\0';
		while((sc=strchr(tmp,1))) *sc='\n';
		reminders[remcount]=(char *)malloc(strlen(tmp)+1);
		strcpy(reminders[remcount],tmp);
		remcount++;
		if(remcount>=MAXREM)
			break;
	}

	fclose(fp);
	return remcount;
}

/*
 * Write the reminders file and free the reminders if asked to
 */
int
write_rem_file(char filename[],int freearray)
{
	FILE *fp;
	char *sc;
	char tmp[MAXREMLEN];
	int n;

	if(!(fp=fopen(filename,"wt")))
		return 0;

	for(n=0;n<remcount;n++) {
		if(reminders[n]) {
			strcpy(tmp,reminders[n]);
			while((sc=strchr(tmp,'\n'))) *sc=1;
			fprintf(fp,"%s\n",tmp);
			if(freearray) {
				free(reminders[n]);
			}
		}
	}
	fclose(fp);
	return n;
}

/*
 * Is the file writable (we create one if it doesn't exist)
 */
int
iswritable(char file[])
{
	FILE *fp;
	if((fp=fopen(file,"a"))) {
		fclose(fp);
		return TRUE;
	}
	return FALSE;
}
