/* Xlrem Login Reminder program for Motif
 * Copyright (C) 1997 George Lebl.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "xlrem.h"

XtAppContext	app;
Widget		toplevel;

int		remcount=0;
char		*reminders[MAXREM];
int		ptrrem[MAXREM];

/*
 * Edit Box Callbacks
 */
void
eboxok_callback(Widget dialog, XtPointer client_data, XtPointer call_data)
{
	*((int *)(client_data))=1;
}
void
eboxcan_callback(Widget dialog, XtPointer client_data, XtPointer call_data)
{
	*((int *)(client_data))=2;
}
void
eboxdes_callback(Widget dialog, XtPointer client_data, XtPointer call_data)
{
	*((int *)(client_data))=10;
}

/*
 * Message Box, simple dialog only with ok and a message
 * only created once
 * APLICATION MODAL
 */
int
ebox(Widget parent, char *tit, char *txt,int blen)
{
	static		Widget dialog=NULL,text_w;
	XmString	title;
	static int	done=0;
	Arg		args[10];
	int		n;
	char		*tmp;

	if(!dialog) {
		n=0;
		XtSetArg(args[n],XmNdeleteResponse,XmDESTROY);n++;
		XtSetArg(args[n],XmNautoUnmanage,False);n++;
		XtSetArg(args[n],XmNdialogStyle,
			XmDIALOG_FULL_APPLICATION_MODAL);n++;
		dialog = XmCreateMessageDialog(parent, "dialog", args, n);
		XtManageChild(
			XmMessageBoxGetChild(dialog,XmDIALOG_OK_BUTTON));
		XtUnmanageChild(
			XmMessageBoxGetChild(dialog,XmDIALOG_HELP_BUTTON));
		XtUnmanageChild(
			XmMessageBoxGetChild(dialog,XmDIALOG_MESSAGE_LABEL));
		n=0;
		XtSetArg(args[n],XmNrows,10); n++;
		XtSetArg(args[n],XmNcolumns,80); n++;
		XtSetArg(args[n],XmNeditMode,XmMULTI_LINE_EDIT); n++;
		XtSetArg(args[n],XmNscrollHorizontal,False); n++;
		XtSetArg(args[n],XmNwordWrap,True); n++;
		text_w=XmCreateScrolledText(dialog,"text_w",args,n);
		XtManageChild(text_w);

		XtAddCallback(dialog,XmNokCallback,eboxok_callback,&done);
		XtAddCallback(dialog,XmNcancelCallback,eboxcan_callback,&done);
		XtAddCallback(dialog,XmNdestroyCallback,eboxdes_callback,&done);
	}
	title=XmStringCreateLocalized(tit);
	XtVaSetValues(dialog,
		XmNdialogTitle,title,
		NULL);
	XmStringFree(title);

	XmTextSetString(text_w,txt);

	XtManageChild(dialog);

	XmProcessTraversal(text_w,XmTRAVERSE_CURRENT);

	done=0;
	XtPopup(XtParent(dialog),XtGrabNone);
	while(!done)
		XtAppProcessEvent(app, XtIMAll);
	/*if it wasn't destroyed close it*/
	if(done!=10) {
		XtPopdown(XtParent(dialog));
		if(done==2)
			return FALSE;
		if((tmp=XmTextGetString(text_w))) {
			if(tmp[0]==0)
				return FALSE;
			strncpy(txt,tmp,blen-1);
			txt[blen-1]='\0';
			XtFree((XtPointer)tmp);
			return TRUE;
		}
		return FALSE;
	/*if it was destroyed set dialog to NULL so*/
	/*that we can use this dialog again*/
	} else {
		dialog=NULL;
		return FALSE;
	}
}

/*
 * Resources and command line options (so that I don't have to do much stuff
 * I just made the help option a resource as well, so theoretically you
 * could set Xlrem-admin*help: True .... which would make the program
 * unusable :)
 */
struct _resrcs {
	Boolean help;
	Boolean system;
} Resrcs;

static XtResource resources[] = {
	{"help","Help",XmRBoolean,sizeof(Boolean),
		XtOffsetOf(struct _resrcs,help),XmRImmediate,False},
	{"system","System",XmRBoolean,sizeof(Boolean),
		XtOffsetOf(struct _resrcs,system),XmRImmediate,False},
};


static XrmOptionDescRec options[] = {
	{"-help","help",XrmoptionNoArg,"True"},
	{"-system","system",XrmoptionNoArg,"True"},
};

/*
 * Recreate ptrrem
 */
void
reptr(void)
{
	int n,actual;
	for(n=0,actual=0;n<remcount;n++)
		if(reminders[n]!=NULL)
			ptrrem[actual++]=n;
}

/*
 * Repack reminders (purge deleted ones)
 */
void
repack(void)
{
	int n,actual;
	for(n=0,actual=0;n<remcount;n++)
		if(reminders[n]!=NULL) {
			ptrrem[actual]=n;
			reminders[actual++]=reminders[n];
		}
	remcount=actual;
}


/*
 * Fill the list with the reminders array
 */
void
filllist(Widget list_w)
{
	char		tmp[80],*sc;
	XmStringTable	rem_list;
	int		n,actual;

	rem_list=(XmStringTable) XtMalloc(remcount*sizeof(XmString));

	actual=0;
	for(n=0;n<remcount;n++) {
		if(reminders[n]!=NULL) {
			strncpy(tmp,reminders[n],79);
			tmp[79]='\0';
			sprintf(tmp,"%-79s",tmp);
			while((sc=strchr(tmp,'\n'))) *sc=' ';
			ptrrem[actual]=n;
			rem_list[actual++]=XmStringCreateLocalized(tmp);
		}
	}

	XtVaSetValues(list_w,
		XmNitems,rem_list,
		XmNitemCount,actual,
		NULL);

	for(n=0;n<actual;n++)
		XmStringFree(rem_list[n]);

	XtFree((XtPointer)rem_list);
}

/*
 * Filemenu callback, only quit (save maybe)
 */
void
file_cb(Widget w,XtPointer client_data,XtPointer call_data)
{
	char	fname[256];
	int	res;

	if((long)client_data==0) {
		if(Resrcs.system==True) {
			res=write_rem_file(SYSREMFILE,TRUE);
		} else {
			sprintf(fname,"%s/"REMFILE,getenv("HOME"));
			res=write_rem_file(fname,TRUE);
		}
		if(res==0) {
			mbox(toplevel,"I can't write to the file!","Error");
			return;
		}
	}
	exit(0);
}


/*we need this available for the edit menu*/
Widget list_w;

/*
 * Edit menu callback function
 * provides Del/Add/Edit to the list
 */
void
edit_cb(Widget w,XtPointer client_data,XtPointer call_data)
{
	int		x,*poslist;
	char		tmp[MAXREMLEN],*sc;
	XmString	str;

	/*delete*/
	if((long)client_data==0) {
		if(!XmListGetSelectedPos(list_w,&poslist,&x))
			return;
		x=*poslist;
		XtFree((XtPointer)poslist);
		free(reminders[ptrrem[x-1]]);
		reminders[ptrrem[x-1]]=NULL;
		XmListDeleteItemsPos(list_w,1,x);
		reptr();
	/*add*/
	} else if((long)client_data==1) {
		if(remcount>=MAXREM)
			repack();
		if(remcount>=MAXREM) {
			mbox(toplevel,"Too many reminders! "
				"Delete some first!","Warning");
			return;
		}
		tmp[0]='\0';
		if(ebox(toplevel,"Add",tmp,MAXREMLEN)) {
			reminders[remcount]=malloc(strlen(tmp)+1);
			strcpy(reminders[remcount],tmp);
			remcount++;
			reptr();
			tmp[79]='\0';
			sprintf(tmp,"%-79s",tmp);
			while((sc=strchr(tmp,'\n'))) *sc=' ';
			str=XmStringCreateLocalized(tmp);
			XmListAddItem(list_w,str,0);
			XmStringFree(str);
		}
	/*edit*/
	} else {
		if(!XmListGetSelectedPos(list_w,&poslist,&x))
			return;
		x=*poslist;
		XtFree((XtPointer)poslist);
		strcpy(tmp,reminders[ptrrem[x-1]]);
		if(ebox(toplevel,"Edit",tmp,MAXREMLEN)) {
			free(reminders[ptrrem[x-1]]);
			reminders[ptrrem[x-1]]=malloc(strlen(tmp)+1);
			strcpy(reminders[ptrrem[x-1]],tmp);
			tmp[79]='\0';
			sprintf(tmp,"%-79s",tmp);
			while((sc=strchr(tmp,'\n'))) *sc=' ';
			str=XmStringCreateLocalized(tmp);
			XmListReplaceItemsPos(list_w,&str,1,x);
			XmStringFree(str);
		}
	}
}


/*
 * Main
 */
int
main(int argc, char *argv[])
{
	Widget		main_w,menubar,menu;
	char		fname[256];
	XmString	file_s,edit_s,exit_s,exitsave_s,del_s,add_s,str;
	Arg		args[5];
	int		n;
	char		tmp[80];

	XtSetLanguageProc(NULL,NULL,NULL);

	toplevel=XtVaAppInitialize(&app,"Xlrem-admin",options,XtNumber(options),
		&argc,argv,NULL,NULL);

	XtGetApplicationResources(toplevel,&Resrcs,
		resources,XtNumber(resources),NULL,0);

	if(Resrcs.help==True) {
		printf("\n(X)(L)ogin(Rem)inders Admin: v"VERSION"\n\n");
		printf(" -system      Edit system ("SYSREMFILE") reminders\n");
		printf("              instead of personal (~/"REMFILE")\n");
		printf(" -help        Show this help!\n\n");
		exit(0);
	}

	if(Resrcs.system==True) {
		if(!iswritable(SYSREMFILE))
			mbox(toplevel,"You don't have permission "
				"to write the system file! No changes "
				"can be saved!","Warning");
		read_rem_file(SYSREMFILE);
	} else {
		sprintf(fname,"%s/"REMFILE,getenv("HOME"));
		remcount=0;
		if(!iswritable(fname))
			mbox(toplevel,"You don't have permission "
				"to write your personal file! No changes "
				"can be saved!","Warning");
		read_rem_file(fname);
	}

	main_w=XtVaCreateManagedWidget("main",
		xmMainWindowWidgetClass,toplevel,
		NULL);

	file_s=XmStringCreateLocalized("File");
	edit_s=XmStringCreateLocalized("Edit");
	menubar=XmVaCreateSimpleMenuBar(main_w,"menubar",
		XmVaCASCADEBUTTON,file_s,'F',
		XmVaCASCADEBUTTON,edit_s,'E',
		NULL);
	XmStringFree(file_s);
	XmStringFree(edit_s);

	exitsave_s=XmStringCreateLocalized("Exit and Save");
	exit_s=XmStringCreateLocalized("Exit");
	menu=XmVaCreateSimplePulldownMenu(menubar,"file_menu",0,file_cb,
		XmVaPUSHBUTTON,exitsave_s,'x',NULL,NULL,
		XmVaPUSHBUTTON,exit_s,'E',NULL,NULL,
		NULL);
	XmStringFree(exit_s);
	XmStringFree(exitsave_s);

	del_s=XmStringCreateLocalized("Delete Item");
	add_s=XmStringCreateLocalized("Add Item");
	edit_s=XmStringCreateLocalized("Edit Item");
	menu=XmVaCreateSimplePulldownMenu(menubar,"edit_menu",1,edit_cb,
		XmVaPUSHBUTTON,del_s,'D',NULL,NULL,
		XmVaSEPARATOR,
		XmVaPUSHBUTTON,add_s,'A',NULL,NULL,
		XmVaPUSHBUTTON,edit_s,'E',NULL,NULL,
		NULL);
	XmStringFree(del_s);
	XmStringFree(add_s);
	XmStringFree(edit_s);

	XtManageChild(menubar);

	n=0;
	XtSetArg(args[n],XmNlistSizePolicy,XmRESIZE_IF_POSSIBLE); n++;
	list_w=XmCreateScrolledList(main_w,"list",args,n);
	XtVaSetValues(list_w,
		XmNvisibleItemCount,10,
		NULL);
	filllist(list_w);

	/* Make sure the list is at least 79 chars long */
	sprintf(tmp,"%-79s","");
	str=XmStringCreateLocalized(tmp);
	XmListAddItem(list_w,str,1);
	XmStringFree(str);

	XtManageChild(list_w);

	XtRealizeWidget(toplevel);

	/* Delete the bogus entry */
	XmListDeleteItemsPos(list_w,1,1);

	XtAppMainLoop(app);

	return 0;
}
