/* Xlrem Login Reminder program for Motif
 * Copyright (C) 1997 George Lebl.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "xlrem.h"

XtAppContext	app;

int		remcount=0;
char		*reminders[MAXREM];

/*
 * Callback routines for reminder dialog (main window!)
 */
void
remdel_callback(Widget dialog, XtPointer client_data, XtPointer call_data)
{
	*((int *)(client_data))=DELANS;
}
void
remskip_callback(Widget dialog, XtPointer client_data, XtPointer call_data)
{
	*((int *)(client_data))=SKIPANS;
}
void
remquit_callback(Widget dialog, XtPointer client_data, XtPointer call_data)
{
	*((int *)(client_data))=QUITANS;
	XtDestroyWidget(XtParent(dialog));
}

/*The variables we need for the dialog*/
Widget	rem_text_w, rem_delpb;
Widget	toplevel;
int	rem_done;

/*
 * Reminder dialog that is drawn on the toplevel (or parent)
 * no need for modality (we are the application)
 * creation routine
 */
void
remcreate(Widget parent, char *tit)
{
	Widget			form,quitpb,pane,skippb;
	XmString		dellab;
	XmString		skiplab;
	XmString		quitlab;
	XtWidgetGeometry	size;
	Widget			*chil;
	int			chil_num;
	Arg			args[10];
	int			n;
	static int 		again=FALSE;

	/*only do this once*/
	if(again)
		return;
	again=TRUE;

	pane=XtVaCreateWidget("pane",xmPanedWindowWidgetClass,parent,
		XmNsashWidth,1,
		XmNsashHeight,1,
		NULL);

	n=0;
	XtSetArg(args[n],XmNrows,10); n++;
	XtSetArg(args[n],XmNcolumns,80); n++;
	XtSetArg(args[n],XmNeditMode,XmMULTI_LINE_EDIT); n++;
	XtSetArg(args[n],XmNscrollHorizontal,False); n++;
	XtSetArg(args[n],XmNwordWrap,True); n++;
	XtSetArg(args[n],XmNeditable,False); n++;
	rem_text_w=XmCreateScrolledText(pane,"text_w",args,n);
	XtManageChild(rem_text_w);

	form=XtVaCreateWidget("form",xmFormWidgetClass,pane,
		XmNfractionBase,5,
		NULL);

	skiplab=XmStringCreateLocalized("Skip");
	dellab=XmStringCreateLocalized("Delete");
	quitlab=XmStringCreateLocalized("Quit");
	skippb=XtVaCreateManagedWidget("skip",
		xmPushButtonGadgetClass,form,
		XmNlabelString,skiplab,
		XmNtopAttachment,XmATTACH_FORM,
		XmNbottomAttachment,XmATTACH_FORM,
		XmNleftAttachment,XmATTACH_POSITION,
		XmNleftPosition,0,
		XmNrightAttachment,XmATTACH_POSITION,
		XmNrightPosition,1,
		XmNshowAsDefault,True,
		XmNdefaultButtonShadowThickness,1,
		NULL);
	rem_delpb=XtVaCreateManagedWidget("delete",
		xmPushButtonGadgetClass,form,
		XmNlabelString,dellab,
		XmNtopAttachment,XmATTACH_FORM,
		XmNbottomAttachment,XmATTACH_FORM,
		XmNleftAttachment,XmATTACH_POSITION,
		XmNleftPosition,2,
		XmNrightAttachment,XmATTACH_POSITION,
		XmNrightPosition,3,
		XmNshowAsDefault,False,
		XmNdefaultButtonShadowThickness,1,
		NULL);
	quitpb=XtVaCreateManagedWidget("quit",
		xmPushButtonGadgetClass,form,
		XmNlabelString,quitlab,
		XmNtopAttachment,XmATTACH_FORM,
		XmNbottomAttachment,XmATTACH_FORM,
		XmNleftAttachment,XmATTACH_POSITION,
		XmNleftPosition,4,
		XmNrightAttachment,XmATTACH_POSITION,
		XmNrightPosition,5,
		XmNshowAsDefault,False,
		XmNdefaultButtonShadowThickness,1,
		NULL);
	XmStringFree(dellab);
	XmStringFree(skiplab);
	XmStringFree(quitlab);
	XtAddCallback(rem_delpb,XmNactivateCallback,
		remdel_callback,&rem_done);
	XtAddCallback(skippb,XmNactivateCallback,
		remskip_callback,&rem_done);
	XtAddCallback(quitpb,XmNactivateCallback,
		remquit_callback,&rem_done);

	XtManageChild(form);

	XtQueryGeometry(form,NULL,&size);
	XtVaSetValues(form,
		XmNpaneMinimum,size.height,
		XmNpaneMaximum,size.height,
		NULL);

	XtManageChild(pane);

	/*I have to do this for some strange reason,
	  shouldn't this work by default or am I missing 
	  something ?? */
	XtOverrideTranslations(form,XtParseTranslationTable(
		"<Key>Return: ManagerGadgetSelect()"));

	XtVaSetValues(parent,XmNtitle,tit,NULL);

	XtVaGetValues(pane,
		XmNchildren,&chil,
		XmNnumChildren,&chil_num,
		NULL);
	while(chil_num-- > 0)
		if(XmIsSash(chil[chil_num]))
			XtVaSetValues(chil[chil_num],
				XmNtraversalOn,False,NULL);

	XtRealizeWidget(parent);

	XmProcessTraversal(skippb,XmTRAVERSE_CURRENT);
}

/*
 * display a text on the reminder window (the main window btw)
 */
int
rem(char *msg, char *tit, int allowdelete)
{
	if(allowdelete)
		XtSetSensitive(rem_delpb,True);
	else
		XtSetSensitive(rem_delpb,False);

	XtVaSetValues(rem_text_w,
		XmNvalue,msg,
		NULL);

	XtVaSetValues(toplevel,XmNtitle,tit,NULL);

	rem_done=0;
	while(!rem_done)
		XtAppProcessEvent(app, XtIMAll);
	return rem_done;
}

/*
 * Resources and command line options (so that I don't have to do much stuff
 * I just made the help option a resource as well, so theoretically you
 * could set Xlrem*help: True .... which would make the program unusable :)
 */
struct _resrcs {
	Boolean help;
	Boolean verbose;
	Boolean nosystem;
	Boolean nopersonal;
} Resrcs;

static XtResource resources[] = {
	{"help","Help",XmRBoolean,sizeof(Boolean),
		XtOffsetOf(struct _resrcs,help),XmRImmediate,False},
	{"verbose","Verbose",XmRBoolean,sizeof(Boolean),
		XtOffsetOf(struct _resrcs,verbose),XmRImmediate,False},
	{"nosystem","NoSystem",XmRBoolean,sizeof(Boolean),
		XtOffsetOf(struct _resrcs,nosystem),XmRImmediate,False},
	{"nopersonal","NoPersonal",XmRBoolean,sizeof(Boolean),
		XtOffsetOf(struct _resrcs,nopersonal),XmRImmediate,False},
};


static XrmOptionDescRec options[] = {
	{"-help","help",XrmoptionNoArg,"True"},
	{"-verbose","verbose",XrmoptionNoArg,"True"},
	{"-nosystem","nosystem",XrmoptionNoArg,"True"},
	{"-nopersonal","nopersonal",XrmoptionNoArg,"True"},
};

/*
 * Main
 */
int
main(int argc, char *argv[])
{
	int		n;
	char		tmp[50],fname[256];
	int		candeletesys;

	XtSetLanguageProc(NULL,NULL,NULL);

	toplevel=XtVaAppInitialize(&app,"Xlrem",options,XtNumber(options),
		&argc,argv,NULL,NULL);

	XtGetApplicationResources(toplevel,&Resrcs,
		resources,XtNumber(resources),NULL,0);

	if(Resrcs.help==True) {
		printf("\n(X)(L)ogin(Rem)inders: v"VERSION"\n\n");
		printf(" -verbose     Bother me with no-reminders popups\n");
		printf(" -nosystem    No system ("SYSREMFILE") reminders\n");
		printf(" -nopersonal  No personal (~/"REMFILE") reminders\n");
		printf(" -help        Show this help!\n\n");
		printf("Or set the Xlrem resources in your .Xdefaults: "
			"Verbose, NoSystem, NoPersonal\n\n");
		exit(0);
	}

	if(Resrcs.nosystem==False) {
		candeletesys=iswritable(SYSREMFILE);
		read_rem_file(SYSREMFILE);
		if(remcount==0) {
			if(Resrcs.verbose) {
				/*create the main window if it isn't yet*/
				remcreate(toplevel,"(X)(L)ogin(Rem)inders");
				mbox(toplevel,
					"There are no system reminders!",
					"xlrem");
			}
		} else {
			for(n=0;n<remcount;n++) {
				/*create the main window if it isn't yet*/
				remcreate(toplevel,"(X)(L)ogin(Rem)inders");
				sprintf(tmp,"System Reminder (%d of %d)",
					n+1,remcount);
				switch(rem(reminders[n],tmp,candeletesys)) {
					case QUITANS:
						exit(0);
						break;
					case DELANS:
						free(reminders[n]);
						reminders[n]=NULL;
				}
				if(candeletesys)
					write_rem_file(SYSREMFILE,FALSE);
				else
					free(reminders[n]);
			}
			if(candeletesys)
				write_rem_file(SYSREMFILE,TRUE);
		}
	}

	if(Resrcs.nopersonal==False) {
		sprintf(fname,"%s/"REMFILE,getenv("HOME"));
		remcount=0;
		read_rem_file(fname);
		if(remcount==0) {
			if(Resrcs.verbose) {
				/*create the main window if it isn't yet*/
				remcreate(toplevel,"(X)(L)ogin(Rem)inders");
				mbox(toplevel,
					"You have no presonal reminders!",
					"xlrem");
			}
			exit(0);
		}
		for(n=0;n<remcount;n++) {
			/*create the main window if it isn't yet*/
			remcreate(toplevel,"(X)(L)ogin(Rem)inders");
			sprintf(tmp,"Reminder (%d of %d)",n+1,remcount);
			switch(rem(reminders[n],tmp,TRUE)) {
				case QUITANS:
					exit(0);
					break;
				case DELANS:
					free(reminders[n]);
					reminders[n]=NULL;
			}
			write_rem_file(fname,FALSE);
		}
		write_rem_file(fname,TRUE);
	}

	return 0;
}
